@@ -1,9 +1,13 @@
+from __future__ import annotations
+from typing import Type
 import json
 import re
 import argparse
+import sys
 from pathlib import Path
 from parse_ecli import pattern
 
+
 def match_ecli(ecli_string):
     """Bestimmt, welche Klasse (=Gericht) auf den ECLI passt."""
 # Reihenfolge der Überprüfung nach geschätzter Häufgikeit der Entscheidungen.
@@ -69,6 +73,102 @@ def match_ecli(ecli_string):
             raise NoValidECLIError("Kein gültiger ECLI!")
 
 
+def search_ecli(ecli_string):
+    decision_list = []
+
+    if re.search(pattern.laender_compiled, ecli_string) is not None:
+        laender_iter = re.finditer(pattern.laender_compiled, ecli_string)
+        for match in laender_iter:
+            try:
+                decision = Decision_Other(match.group())
+                decision.parse_ecli(match)
+                decision.position = match.start()
+                decision_list.append(decision)
+            except (NoValidECLIError, InValidAZError, InValidCourtError, UnknownAZError):
+                # Es kann sein, dass ein ECLI gematcht wurde, der aber dennoch ungültig ist (z.B. im Az)
+                # dies wird im searchmode jedoch ignoriert
+                pass
+
+    if re.search(pattern.bgh_compiled, ecli_string) is not None:
+        bgh_iter = re.finditer(pattern.bgh_compiled, ecli_string)
+        for match in bgh_iter:
+            try:
+                decision = Decision_BGH(match.group())
+                decision.parse_ecli(match)
+                decision.position = match.start()
+                decision_list.append(decision)
+            except (NoValidECLIError, InValidAZError, InValidCourtError, UnknownAZError):
+                pass
+
+    if re.search(pattern.bverfg_compiled, ecli_string) is not None:
+        bverfg_iter = re.finditer(pattern.bverfg_compiled, ecli_string)
+        for match in bverfg_iter:
+            try:
+                decision = Decision_BVerfG(match.group())
+                decision.parse_ecli(match)
+                decision.position = match.start()
+                decision_list.append(decision)
+            except (NoValidECLIError, InValidAZError, InValidCourtError, UnknownAZError):
+                pass
+
+    if re.search(pattern.bverwg_compiled, ecli_string) is not None:
+        bverwg_iter = re.finditer(pattern.bverwg_compiled, ecli_string)
+        for match in bverwg_iter:
+            try:
+                decision = Decision_BVerwG(match.group())
+                decision.parse_ecli(match)
+                decision.position = match.start()
+                decision_list.append(decision)
+            except (NoValidECLIError, InValidAZError, InValidCourtError, UnknownAZError):
+                pass
+
+    if re.search(pattern.bag_compiled, ecli_string) is not None:
+        bag_iter = re.finditer(pattern.bag_compiled, ecli_string)
+        for match in bag_iter:
+            try:
+                decision = Decision_BAG(match.group())
+                decision.parse_ecli(match)
+                decision.position = match.start()
+                decision_list.append(decision)
+            except (NoValidECLIError, InValidAZError, InValidCourtError, UnknownAZError):
+                pass
+
+    if re.search(pattern.bsg_compiled, ecli_string) is not None:
+        bsg_iter = re.finditer(pattern.bsg_compiled, ecli_string)
+        for match in bsg_iter:
+            try:
+                decision = Decision_BSG(match.group())
+                decision.parse_ecli(match)
+                decision.position = match.start()
+                decision_list.append(decision)
+            except (NoValidECLIError, InValidAZError, InValidCourtError, UnknownAZError):
+                pass
+
+    if re.search(pattern.bfh_compiled, ecli_string) is not None:
+        bfh_iter = re.finditer(pattern.bfh_compiled, ecli_string)
+        for match in bfh_iter:
+            try:
+                decision = Decision_BFH(match.group())
+                decision.parse_ecli(match)
+                decision.position = match.start()
+                decision_list.append(decision)
+            except (NoValidECLIError, InValidAZError, InValidCourtError, UnknownAZError):
+                pass
+
+    if re.search(pattern.bpatg_compiled, ecli_string) is not None:
+        bpatg_iter = re.finditer(pattern.bpatg_compiled, ecli_string)
+        for match in bpatg_iter:
+            try:
+                decision = Decision_BPatG(match.group())
+                decision.parse_ecli(match)
+                decision.position = match.start()
+                decision_list.append(decision)
+            except (NoValidECLIError, InValidAZError, InValidCourtError, UnknownAZError):
+                pass
+
+    return decision_list
+
+
 class Decision:
     """Grundklasse für alle Entscheidungstypen"""
     # Jedem ECLI sind verschiedene Typen von Entscheidungen zugeordnet, weil sich die
@@ -78,7 +178,7 @@ class Decision:
     def __init__(self, ecli_string):
         """Initialize ECLI"""
         self.ecli = ecli_string
-
+        self.position = 0
         self.court_data = {
             # Diese Daten sollen mithilfe des ECLI aufgefüllt werden.
             # Nicht alle werden von allen Entscheidungstypen genutzt
@@ -174,6 +274,7 @@ class Decision_BVerfG(Decision):
         return url
 
     def parse_ecli(self, match):
+
         """Füllt die Daten für BVerfG-Entscheidungen auf Grundlage des ECLI aus"""
         self.court_data["court"][1] = "BVerfG" # Dies steht fest, da sonst kein Regex-Match
         self.court_data["decisiontype"][1] = self.loaded_data["bverfg_decisiontype"][match.group("type")]
@@ -226,6 +327,7 @@ class Decision_BGH(Decision):
 
     def parse_ecli(self, match):
 
+
         self.court_data["court"][1] = "BGH"
         self.court_data["decisiontype"][1] = self.loaded_data["bgh_decisiontype"][match.group("type")]
         self.court_data["date"][1] = match.group("date")[0:2] + "." + match.group("date")[2:4] + "." + match.group("year")
@@ -267,13 +369,15 @@ class Decision_BVerwG(Decision):
 
     def parse_ecli(self, match):
 
+
         self.court_data["court"][1] = "BVerwG"
         self.court_data["decisiontype"][1] = self.loaded_data["bverwg_decisiontype"][match.group("type")]
         self.court_data["date"][1] = match.group("date")[0:2] + "." + match.group("date")[2:4] + "." + match.group("year")
         self.court_data["collision"][1] = super().check_collision(match.group("collision"))
         try:
-            self.court_data["az"][1] = (match.group("azbody") + " " + self.loaded_data["bverwg_az"][match.group("azreg")]
+            azstring = (match.group("azbody") + " " + self.loaded_data["bverwg_az"][match.group("azreg")]
                                             + " " + match.group("aznumber").lstrip("0") + "." + match.group("azyear"))
+            self.court_data["az"][1] = ' '.join(azstring.split())
             self.court_data["decision_explain"][1] = self.loaded_data["bverwg_explain"][match.group("azreg")]
         except KeyError as e:
             raise InValidAZError(f"Ungültiges Registerzeichen: {match.group('azreg')}") from e
@@ -291,19 +395,22 @@ class Decision_BFH(Decision):
 
     def parse_ecli(self, match):
 
+
         self.court_data["court"][1] = "BFH"
         self.court_data["decisiontype"][1] = self.loaded_data["bfh_decisiontype"][match.group("type")]
         self.court_data["date"][1] = match.group("date")[0:2] + "." + match.group("date")[2:4] + "." + match.group("year")
         self.court_data["collision"][1] = super().check_collision(match.group("collision"))
         azbody = super().check_azpart_empty(match.group("azbody"))
         self.court_data["bodytype"][1] = self.determine_body(azbody, match.group("azreg"))
-        self.court_data["az"][1] = (azbody + " " + match.group("azreg")
+        azstring = (azbody + " " + match.group("azreg")
                                         + " " + match.group("aznumber").lstrip("0") + "/" + match.group("azyear"))
+        self.court_data["az"][1] = ' '.join(azstring.split())
         self.court_data["decision_explain"][1] = self.loaded_data["bfh_explain"][match.group("azreg")]
 
 class Decision_BPatG(Decision):
     def parse_ecli(self, match):
 
+
         self.court_data["court"][1] = "BPatG"
         self.court_data["decisiontype"][1] = self.loaded_data["bpatg_decisiontype"][match.group("type")]
         self.court_data["date"][1] = match.group("date")[0:2] + "." + match.group("date")[2:4] + "." + match.group("year")
@@ -311,30 +418,43 @@ class Decision_BPatG(Decision):
         suffix = super().check_azpart_empty(match.group("azsuffix"))
         if suffix != '':
             suffix = " (" + suffix + ")"
-        self.court_data["az"][1] = (match.group("azbody") + " " + self.loaded_data["bpatg_az"][match.group("azreg")]
+        azstring = (match.group("azbody") + " " + self.loaded_data["bpatg_az"][match.group("azreg")]
                                         + " " + match.group("aznumber").lstrip("0") + "/" + match.group("azyear") + suffix)
+        self.court_data["az"][1] = ' '.join(azstring.split())
         self.court_data["decision_explain"][1] = self.loaded_data["bpatg_explain"][match.group("azreg")]
 
 class Decision_BAG(Decision):
     def parse_ecli(self, match):
 
+
         self.court_data["court"][1] = "BAG"
         # Hier kann auf BVerwG zurückgegriffen werden, da Schnittmenge identisch
         self.court_data["decisiontype"][1] = self.loaded_data["bverwg_decisiontype"][match.group("type")]
         self.court_data["date"][1] = match.group("date")[0:2] + "." + match.group("date")[2:4] + "." + match.group("year")
         self.court_data["collision"][1] = super().check_collision(match.group("collision"))
         azbody = super().check_azpart_empty(match.group("azbody"))
-        self.court_data["az"][1] = (azbody + " " + match.group("azreg")
+        azstring = (azbody + " " + match.group("azreg")
                                         + " " + match.group("aznumber").lstrip("0") + "/" + match.group("azyear"))
+        self.court_data["az"][1] = ' '.join(azstring.split())
         self.court_data["decision_explain"][1] = self.loaded_data["bag_explain"][match.group("azreg")]
 
 class Decision_BSG(Decision):
+    def determine_body(self, azbody, azreg):
+
+        if azbody == "GS":
+            return "Großer Senat"
+        else:
+            return azbody + ". Senat"
+
+
     def parse_ecli(self, match):
 
+
         self.court_data["court"][1] = "BSG"
         self.court_data["decisiontype"][1] = self.loaded_data["bpatg_decisiontype"][match.group("type")]
         self.court_data["date"][1] = match.group("date")[0:2] + "." + match.group("date")[2:4] + "." + match.group("year")
         self.court_data["collision"][1] = super().check_collision(match.group("collision"))
+        self.court_data["bodytype"][1] = self.determine_body(match.group("azbody"), match.group("azreg"))
         azinstance = super().check_azpart_empty(match.group("azinstance"))
         if azinstance == '':
             if match.group("azbody") != "GS":
@@ -343,11 +463,13 @@ class Decision_BSG(Decision):
                 raise InValidAZError("Kein führendes B im Aktenzeichen (nur bei GS-Verfahren zulässig!)")
 
         azreg = super().check_azpart_empty(match.group("azreg"))
+        if azreg == "UEG":
+            azreg = "ÜG"
         if azreg == '':
             if match.group("azbody") != "GS":
                 raise InValidAZError("Kein Sachgebiet im Aktenzeichen (nur bei GS-Verfahren zulässig!)")
             else:
-                self.court_data["decision_explain"][1] = "Großer Senat"
+                self.court_data["decision_explain"][1] = "Verfahren vor dem Großen Senat"
         else:
             self.court_data["decision_explain"][1] = self.loaded_data["bsg_explain"][match.group("azreg")]
 
@@ -359,9 +481,11 @@ class Decision_BSG(Decision):
         else:
             self.court_data["register_explain"][1] = self.loaded_data["bsg_register_explain"][match.group("azregister")]
 
-        self.court_data["az"][1] = (azinstance + " " + match.group("azbody") + " " + azreg + " "
-                                        + match.group("aznumber").lstrip("0") + "/" + match.group("azyear") + " " + azregister).strip()
+        azstring = (azinstance + " " + match.group("azbody") + " " + azreg + " "
+                                        + match.group("aznumber").lstrip("0") + "/" + match.group("azyear") + " " + azregister)
 
+        # Entfernt doppelte Leerzeichen und leading/trailing
+        self.court_data["az"][1] = ' '.join(azstring.split())
 
 
 class Decision_Other(Decision):
@@ -395,6 +519,8 @@ class Decision_Other(Decision):
                 return "v"
         elif match.group("court").startswith("FG"):
             return "f"
+        else:
+            return "unknown"
 
     def parse_ecli_az_sozg(self, match):
         if az_match := re.match(pattern.sozg_az, match.group("az"), flags=re.VERBOSE):
@@ -425,12 +551,34 @@ class Decision_Other(Decision):
             if azreg_sta != '':
                 azreg_sta = self.loaded_data["ordentliche_az"][azreg_sta]
             azsuffix = super().check_azpart_empty(az_match.group("azsuffix"))
+            azreg = az_match.group("azreg")
 
-            az = (azprefix + " " + azbody + " " + self.loaded_data["ordentliche_az"][az_match.group("azreg").replace(".","")]
+            if azreg.replace(".","") in self.loaded_data["ordentliche_az"]:
+                if az_match.group("azreg").replace(".","") in self.loaded_data["ordentliche_explain"]:
+                    decision_explain = self.loaded_data["ordentliche_explain"][az_match.group("azreg").replace(".","")]
+                else:
+                    # Dies bedeutet im Zweifel keinen ungültigen ECLI, deshalb keine Exception, sondern nur
+                    # Information
+                    decision_explain = "Unbekanntes Registerzeichen!"
+                azreg = self.loaded_data["ordentliche_az"][azreg.replace(".","")]
+            elif azreg.startswith(("A", "B", "C", "D")) and azreg.replace(".","")[1:] in self.loaded_data["ordentliche_az"]:
+                # Wenn dies zutrifft, haben wir es höchstwahrscheinlich mit einer Abteilung/Kammer mit alphanumerischer
+                # Bezeichnung zu tun wie ECLI:DE:AGHHAL:2019:0816.327CCS76.18.0A (Abteilung 327c) oder ECLI:DE:LGD:2019:0808.4CO88.17.0A
+                # (4c Zivilkammer)
+                if az_match.group("azreg").replace(".","")[1:] in self.loaded_data["ordentliche_explain"]:
+                    decision_explain = self.loaded_data["ordentliche_explain"][az_match.group("azreg").replace(".","")[1:]]
+                else:
+                    decision_explain = "Unbekanntes Registerzeichen!"
+                azbody = azbody + azreg[0].lower()
+                azreg = self.loaded_data["ordentliche_az"][azreg.replace(".","")[1:]]
+            else:
+                decision_explain = "Unbekanntes Registerzeichen!"
+            az = (azprefix + " " + azbody + " " + azreg
                     + " " + azbody_sta + " " + azreg_sta + " "  + az_match.group("aznumber").lstrip("0") + "/" + az_match.group("azyear")
                     + " " + azsuffix).strip() # Strip bezieht sich auf den gesamten String, der az zugewiesen wird!
             az = re.sub(r"\s\s+" , " ", az)
-            decision_explain = self.loaded_data["ordentliche_explain"][az_match.group("azreg").replace(".","")]
+
+
             return az, decision_explain
         else:
             raise InValidAZError("Ungültiges Aktenzeichen!")
@@ -438,46 +586,58 @@ class Decision_Other(Decision):
 
     def parse_ecli_az_verwg(self, match):
         decision_explain = ''
+        register_explain = ''
         if az_match := re.match(pattern.verwg_az, match.group("az"), flags=re.VERBOSE):
             azregister = super().check_azpart_empty(az_match.group("azregister"))
             if azregister != '':
                 azregister = "." + azregister
-                decision_explain = self.loaded_data["verwg_register_explain"][az_match.group("azregister")]
+                if "VGHH" in match.group("court"):
+                    register_explain = self.loaded_data["verwg_register_explain_hamburg"][az_match.group("azregister")]
+                else:
+                    register_explain = self.loaded_data["verwg_register_explain"][az_match.group("azregister")]
             azhessen = super().check_azpart_empty(az_match.group("azhessen"))
             # In Hessen steht wird den Az. eine Kurzbezeichnung des Gericht angefügt,
             # z.B. KS für VG Kassel: ECLI:DE:VGKASSE:2020:0406.3L348.20.KS.00
             if azhessen != '':
                 azhessen = "." + azhessen
-            azprefix =super().check_azpart_empty(az_match.group("azprefix"))
+
+            azprefix = super().check_azpart_empty(az_match.group("azprefix"))
             if azprefix != '':
                 azprefix = azprefix + " "
+
             az = (azprefix + az_match.group("azbody") + " " + az_match.group("azreg") + " "
                     + az_match.group("aznumber").lstrip("0") + "/" + az_match.group("azyear") + azhessen + azregister).strip()
-            if az_match.group("azreg") in self.loaded_data["verwg_explain"]:
-                decisiontype = self.loaded_data["verwg_explain"][az_match.group("azreg")]
+
+            if "VGHH" in match.group("court"):
+                    decision_explain = self.loaded_data["verwg_explain_hamburg"][az_match.group("azreg")]
             else:
-                decisiontype = "Unbekanntes Registerzeichen!"
-            return az, decision_explain, decisiontype
+                if az_match.group("azreg") in self.loaded_data["verwg_explain"]:
+                    decision_explain = self.loaded_data["verwg_explain"][az_match.group("azreg")]
+                else:
+                    decision_explain = "Unbekanntes Registerzeichen!"
+
+            return az, decision_explain, register_explain
         else:
             raise InValidAZError("Ungültiges Aktenzeichen!")
 
     def parse_ecli_az_verwg_bayern(self, match):
         # Bayerische Az der Verwaltungsgerichte weichen vom üblichen Schema ab und haben daher eine eigene Funktion
         decision_explain = ''
+        register_explain = ''
         if az_match := re.match(pattern.verwg_az_bayern, match.group("az"), flags=re.VERBOSE):
             azregister = super().check_azpart_empty(az_match.group("azregister"))
             if azregister != '':
                 azregister = "." + azregister
-                decision_explain = self.loaded_data["verwg_register_explain"][az_match.group("azregister")]
+                register_explain = self.loaded_data["verwg_register_explain"][az_match.group("azregister")]
             az = (az_match.group("azbayern") + " " + az_match.group("azbody") + " " + az_match.group("azreg")
                     + " " + az_match.group("azyear") + "." + az_match.group("aznumber") + azregister).strip()
-            if az_match.group("azreg") in self.loaded_data["verwg_explain"]:
-                decisiontype = self.loaded_data["verwg_explain"][az_match.group("azreg")]
+            if az_match.group("azreg") in self.loaded_data["verwg_explain_bayern"]:
+                decision_explain = self.loaded_data["verwg_explain_bayern"][az_match.group("azreg")]
             else:
                 # Dies bedeutet im Zweifel keinen ungültigen ECLI, deshalb keine Exception, sondern nur
                 # Information
-                decisiontype = "Unbekanntes Registerzeichen!"
-            return az, decision_explain, decisiontype
+                decision_explain = "Unbekanntes Registerzeichen!"
+            return az, decision_explain, register_explain
         else:
             raise InValidAZError("Ungültiges Aktenzeichen!")
 
@@ -493,7 +653,7 @@ class Decision_Other(Decision):
                 decisiontype = "Unbekanntes Registerzeichen!"
             return az, decision_explain, decisiontype
         else:
-            raise InValidAZError(f"Ungültiges Aktenzeichen!: {match.group('az')}")
+            raise InValidAZError(f"Ungültiges oder unbekanntes Aktenzeichen!: {match.group('az')}")
 
     def parse_ecli(self, match):
         self.valid_court(match)
@@ -506,23 +666,30 @@ class Decision_Other(Decision):
         # erfasst zu werden.
         jurisdiction = self.determine_jurisdiction(match)
         # Das Aktenzeichen wird für jede Gerichtsbarkeit anders gebildet, also zunächst bestimmen, welche hier vorliegt.
-        if jurisdiction == "o":
-            self.court_data["az"][1], self.court_data["decision_explain"][1] = self.parse_ecli_az_ordentliche(match)
-        elif jurisdiction == "s":
-            self.court_data["az"][1], self.court_data["decision_explain"][1] = self.parse_ecli_az_sozg(match)
-        elif jurisdiction == "a":
-            self.court_data["az"][1], self.court_data["decision_explain"][1] = self.parse_ecli_az_arbg(match)
-        elif jurisdiction == "v":
-            self.court_data["az"][1], self.court_data["decision_explain"][1], self.court_data["decisiontype"][1]\
-             = self.parse_ecli_az_verwg(match)
-        elif jurisdiction == "vbay":
-            self.court_data["az"][1], self.court_data["decision_explain"][1], self.court_data["decisiontype"][1]\
-             = self.parse_ecli_az_verwg_bayern(match)
-        elif jurisdiction == "f":
-            self.court_data["az"][1], self.court_data["decision_explain"][1], self.court_data["decisiontype"][1]\
-             = self.parse_ecli_az_fg(match)
-        else:
-            self.court_data["az"][1] = match.group("az")
+        try:
+            if jurisdiction == "o":
+                self.court_data["az"][1], self.court_data["decision_explain"][1] = self.parse_ecli_az_ordentliche(match)
+            elif jurisdiction == "s":
+                self.court_data["az"][1], self.court_data["decision_explain"][1] = self.parse_ecli_az_sozg(match)
+            elif jurisdiction == "a":
+                self.court_data["az"][1], self.court_data["decision_explain"][1] = self.parse_ecli_az_arbg(match)
+            elif jurisdiction == "v":
+                self.court_data["az"][1], self.court_data["decision_explain"][1], self.court_data["register_explain"][1]\
+                 = self.parse_ecli_az_verwg(match)
+            elif jurisdiction == "vbay":
+                self.court_data["az"][1], self.court_data["decision_explain"][1], self.court_data["register_explain"][1]\
+                 = self.parse_ecli_az_verwg_bayern(match)
+            elif jurisdiction == "f":
+                self.court_data["az"][1], self.court_data["decision_explain"][1], self.court_data["register_explain"][1]\
+                 = self.parse_ecli_az_fg(match)
+            else:
+                self.court_data["az"][1] = match.group("az")
+        except (NoValidECLIError, InValidAZError, InValidCourtError) as e:
+            if jurisdiction == "o":
+                self.court_data["az"][1], self.court_data["decision_explain"][1] = match.group("az"), ""
+                raise UnknownAZError("Aktenzeichen konnte nicht analysiert werden und wird unverändert aus ECLI übernommen")
+            else:
+                raise e
 
 
 class NoECLIError(Exception):
@@ -537,6 +704,9 @@ class InValidCourtError(Exception):
 class InValidAZError(Exception):
     pass
 
+class UnknownAZError(Exception):
+    pass
+
 
 ##################
 # Hauptprogramm
@@ -551,6 +721,127 @@ def exception_print(e):
     print(e)
     print("#\n#\n####################################################\n\n")
 
+def get_input(input_file_string, batchmode=False):
+    input_file_path = Path(input_file_string)
+    ecli_list = []
+    try:
+        with open(input_file_path, encoding="utf8") as input_file:
+            if batchmode:
+                for index, line in enumerate(input_file):
+                    # Alle Zeilen, die nicht mit ECLI:DE: beginnen, werden ignoriert.
+                    if line.upper().startswith("ECLI:DE:"):
+                        ecli_list.append([line, index+1])
+                return ecli_list
+            else:
+                input_file_content = input_file.read()
+                ecli_list.append([input_file_content.upper(),])
+                return ecli_list
+    except FileNotFoundError:
+        print(f"Datei {input_file_string} existiert nicht!")
+        sys.exit(0)
+
+def write_to_file(match_list, output_file_arg, rawmode=False):
+    output_file_path = Path(output_file_arg)
+    if output_file_path.exists():
+        overwrite = input("Datei existiert bereits. Überschreiben? (j/n): ")
+        if overwrite.lower() == 'j' or overwrite.lower() == 'y':
+            with open(output_file_path, mode="w", encoding="utf-8") as f:
+                for decision in match_list:
+                    decision.output_decision(rawmode, f)
+                print("Datei überschrieben")
+        else:
+            print("Abbruch")
+            sys.exit(0)
+    else:
+        try:
+            with open(output_file_path, mode="w", encoding="utf-8") as f:
+                for decision in match_list:
+                    decision.output_decision(rawmode, f)
+        except FileNotFoundError as e:
+            print(f"Dateiname/-pfad ungültig: {e}")
+
+def analyse_file_batch(ecli_list, silent=False):
+    if silent:
+        pass
+    else:
+        print(f"{len(ecli_list)} Zeilen gefunden, die mit ECLI:DE: beginnen.")
+    match_list = []
+    for ecli_list_entry in ecli_list:
+        try:
+            index = ecli_list_entry[1]
+            # index muss vor my_decision zugewiesen werden, sonst werden bei exception
+            # falsche Zeilen ausgegeben!
+            my_decision = match_ecli(ecli_list_entry[0])
+            match_list.append(my_decision)
+        except (NoValidECLIError, InValidAZError, InValidCourtError) as e:
+            if silent:
+                pass
+            else:
+                print(f"Ignoriere ungültigen ECLI in Zeile {index}: {e}.")
+        except (UnknownAZError) as e:
+            if silent:
+                pass
+            else:
+                print(f"Warnung: ECLI in Zeile {index}: {e}.")
+    if silent:
+        pass
+    else:
+        print(f"Insgesamt waren {len(match_list)} von {len(ecli_list)} erkannten ECLI gültig.")
+    return match_list
+
+def analyse_file_search(ecli_list, silent=False):
+    try:
+        match_list = search_ecli(ecli_list[0][0])
+    except (NoValidECLIError, InValidAZError, InValidCourtError, UnknownAZError) as e:
+        pass
+    if silent:
+        pass
+    else:
+        print(f"{len(match_list)} gültige ECLI gefunden.")
+        print("Positionen in Datei:",end=" ")
+        position_list = []
+        for decision in match_list:
+            position_list.append(decision.position)
+            position_list.sort()
+        print(position_list)
+    return match_list
+
+
+def read_from_file_mode(input_file_string, output_file_string=None, raw=False, batch=False, silent=False):
+    ecli_list = []
+    match_list = []
+    ecli_list = get_input(input_file_string, batch)
+    if len(ecli_list) == 0:
+        if silent:
+            pass
+        else:
+            print("Keine ECLI gefunden!")
+    else:
+        if batch:
+            match_list = analyse_file_batch(ecli_list, silent)
+        else:
+            match_list = analyse_file_search(ecli_list, silent)
+
+        if output_file_string is not None:
+            write_to_file(match_list, output_file_string, raw)
+        else:
+            for decision in match_list:
+                # print(f"Position: {decision.start()}")
+                decision.output_decision(raw)
+
+
+def commandline_mode(ecli_string, output_file, raw=False):
+    try:
+        my_decision = match_ecli(ecli_string)
+        if output_file is not None:
+            match_list = [my_decision]
+            write_to_file(match_list, output_file, raw)
+        else:
+            my_decision.output_decision(raw)
+    except (NoValidECLIError, InValidAZError, InValidCourtError) as e:
+        exception_print(e)
+
+
 
 def main_func():
     # Der folgende Abschnitt dient zur Erfassung der Eingabe über die Kommandozeile
@@ -564,56 +855,23 @@ def main_func():
                         help='Angabe einer Datei, in welche die Ausgabe geschrieben werden soll. Erzeugt mit -r eine .csv'))
     (parser.add_argument('-r', '--raw', action='store_true',
                         help='Ausgabe der analysierten Bestandteile ohne weitere Beschriftung, durch Semikolon getrennt'))
-    (parser.add_argument('-v', '--version', action='version', version='%(prog)s 0.9.3'))
+    (parser.add_argument('-b', '--batch', action='store_true',
+                        help='Bei Angabe einer Input-Datei wird nicht nach ECLI gesucht, sondern jede Zeile als ECLI behandelt'))
+    (parser.add_argument('-s', '--silent', action='store_true',
+                        help='Unterdrückt Warnungen, wenn eine Datei zum Einlesen angegeben wurde'))
+    (parser.add_argument('-v', '--version', action='version', version='%(prog)s 0.9.4'))
     args, args2 = parser.parse_known_args()
 
-    ecli_list = []
-    match_list = []
 
     if args.input_file is not None:
         # Falls mit -i eine Datei angegeben wurde:
-        input_file_string = Path(args.input_file)
-        try:
-            with open(input_file_string) as input_file:
-                lines = input_file.readlines()
-                for line in lines:
-                    # Alle Zeilen, die nicht mit ECLI:DE: beginnen, werden ignoriert.
-                    if line.upper().startswith("ECLI:DE:"):
-                        ecli_list.append(line)
-        except FileNotFoundError:
-            print(f"Datei {input_file_string} existiert nicht!")
+        read_from_file_mode(args.input_file, args.output_file, args.raw, args.batch, args.silent)
     else:
+        # Falls direkt von der Kommandozeile gelesen wird.
         parser2 = argparse.ArgumentParser()
         parser2.add_argument('ecli', metavar='ECLI', help='Der zu überprüfende ECLI')
         # Ansonsten einzelnen ECLI direkt von der Kommandozeile
         args2 = parser2.parse_args(args2)
-        ecli_string = args2.ecli.upper().strip(' .:/()')
-        ecli_list.append(ecli_string)
-
-
-    if len(ecli_list) == 0:
-        print("Keine ECLI gefunden!")
-    else:
-        for ecli_string in ecli_list:
-            try:
-                my_decision = match_ecli(ecli_string)
-                match_list.append(my_decision)
-            except (NoValidECLIError, InValidAZError, InValidCourtError) as e:
-                exception_print(e)
-
-        if args.output_file is not None:
-            output_file_string = Path(args.output_file)
-            if output_file_string.exists():
-                overwrite = input("Datei existiert bereits. Überschreiben? (j/n): ")
-                if overwrite.lower() == 'j':
-                    with open(args.output_file, mode="w", encoding="utf-8") as f:
-                        for decision in match_list:
-                            decision.output_decision(args.raw, f)
-                        print("Datei überschrieben")
-            else:
-                with open(args.output_file, mode="w", encoding="utf-8") as f:
-                    for decision in match_list:
-                        decision.output_decision(args.raw, f)
-        else:
-            for decision in match_list:
-                decision.output_decision(args.raw)
+        ecli_string = args2.ecli.upper()
+        ecli_string = ecli_string.strip()
+        commandline_mode(ecli_string, args.output_file, args.raw)
